defmodule Diffuser.Application do
  # See https://hexdocs.pm/elixir/Application.html
  # for more information on OTP Applications
  @moduledoc false

  use Application

  @impl true
  def start(_type, _args) do
    children = [
      # Start the Ecto repository
      Diffuser.Repo,
      # Start the Telemetry supervisor
      DiffuserWeb.Telemetry,
      # Start the PubSub system
      {Phoenix.PubSub, name: Diffuser.PubSub},
      # Start the Endpoint (http/https)
      DiffuserWeb.Endpoint,
      # Start a worker by calling: Diffuser.Worker.start_link(arg)
      # {Diffuser.Worker, arg}
      {Task.Supervisor, name: Diffuser.Generator.PromptRequestSupervisor}
    ]

    # See https://hexdocs.pm/elixir/Supervisor.html
    # for other strategies and supported options
    opts = [strategy: :one_for_one, name: Diffuser.Supervisor]
    Supervisor.start_link(children, opts)
  end

  # Tell Phoenix to update the endpoint configuration
  # whenever the application is updated.
  @impl true
  def config_change(changed, _new, removed) do
    DiffuserWeb.Endpoint.config_change(changed, removed)
    :ok
  end
end
